@@ -6,24 +6,28 @@
 import copy
 import gc
 import sys
+from collections import OrderedDict
 
+import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 
 from ...extern import six
 from ...io import fits
-from ...tests.helper import (pytest, assert_follows_unicode_guidelines,
-                             ignore_warnings)
+from ...tests.helper import (assert_follows_unicode_guidelines,
+                             ignore_warnings, catch_warnings)
 from ...utils.data import get_pkg_data_filename
 from ... import table
 from ... import units as u
 from .conftest import MaskedTable
 
+from ...extern.six.moves import zip, range, cStringIO as StringIO
+
 try:
     with ignore_warnings(DeprecationWarning):
         # Ignore DeprecationWarning on pandas import in Python 3.5--see
         # https://github.com/astropy/astropy/issues/4380
-        import pandas
+        import pandas  # pylint: disable=W0611
 except ImportError:
     HAS_PANDAS = False
 else:
@@ -134,8 +138,8 @@ class TestSetTableColumn(SetupData):
         self._setup(table_types)
         t = table_types.Table()
 
-        t['aa'] = np.array([1,2,3]) * u.m
-        assert np.all(t['aa'] == np.array([1,2,3]))
+        t['aa'] = np.array([1, 2, 3]) * u.m
+        assert np.all(t['aa'] == np.array([1, 2, 3]))
         assert t['aa'].unit == u.m
 
         t['bb'] = 3 * u.m
@@ -179,8 +183,8 @@ class TestSetTableColumn(SetupData):
         assert np.all(t['f'] == 10)
 
         # Add a column from a Quantity
-        t['g'] = np.array([1,2,3]) * u.m
-        assert np.all(t['g'].data == np.array([1,2,3]))
+        t['g'] = np.array([1, 2, 3]) * u.m
+        assert np.all(t['g'].data == np.array([1, 2, 3]))
         assert t['g'].unit == u.m
 
         # Add a column from a (scalar) Quantity
@@ -260,7 +264,7 @@ class TestNewFromColumns():
         t = table_types.Table(cols)
         assert np.all(t['a'].data == np.array([1, 2, 3]))
         assert np.all(t['b'].data == np.array([4, 5, 6], dtype=np.float32))
-        assert type(t['b'][1]) == np.float32
+        assert type(t['b'][1]) is np.float32
 
     def test_from_np_array(self, table_types):
         cols = [table_types.Column(name='a', data=np.array([1, 2, 3], dtype=np.int64),
@@ -269,8 +273,8 @@ class TestNewFromColumns():
         t = table_types.Table(cols)
         assert np.all(t['a'] == np.array([1, 2, 3], dtype=np.float64))
         assert np.all(t['b'] == np.array([4, 5, 6], dtype=np.float32))
-        assert type(t['a'][1]) == np.float64
-        assert type(t['b'][1]) == np.float32
+        assert type(t['a'][1]) is np.float64
+        assert type(t['b'][1]) is np.float32
 
     def test_size_mismatch(self, table_types):
         cols = [table_types.Column(name='a', data=[1, 2, 3]),
@@ -287,6 +291,7 @@ class TestNewFromColumns():
         t = table_types.Table([c, d])
         assert t.colnames == ['c', 'col1']
 
+
 @pytest.mark.usefixtures('table_types')
 class TestReverse():
 
@@ -333,6 +338,13 @@ class TestColumnAccess():
         with pytest.raises(KeyError):
             t['b']  # column does not exist
 
+    def test_itercols(self, table_types):
+        names = ['a', 'b', 'c']
+        t = table_types.Table([[1], [2], [3]], names=names)
+        for name, col in zip(names, t.itercols()):
+            assert name == col.name
+            assert isinstance(col, table_types.Column)
+
 
 @pytest.mark.usefixtures('table_types')
 class TestAddLength(SetupData):
@@ -416,6 +428,40 @@ class TestAddPosition(SetupData):
         assert t.columns.keys() == ['c', 'a', 'b']
 
 
+@pytest.mark.usefixtures('table_types')
+class TestAddName(SetupData):
+
+    def test_override_name(self, table_types):
+        self._setup(table_types)
+        t = table_types.Table()
+
+        # Check that we can override the name of the input column in the Table
+        t.add_column(self.a, name='b')
+        t.add_column(self.b, name='a')
+        assert t.columns.keys() == ['b', 'a']
+        # Check that we did not change the name of the input column
+        assert self.a.info.name == 'a'
+        assert self.b.info.name == 'b'
+
+        # Now test with an input column from another table
+        t2 = table_types.Table()
+        t2.add_column(t['a'], name='c')
+        assert t2.columns.keys() == ['c']
+        # Check that we did not change the name of the input column
+        assert t.columns.keys() == ['b', 'a']
+
+        # Check that we can give a name if none was present
+        col = table_types.Column([1, 2, 3])
+        t.add_column(col, name='c')
+        assert t.columns.keys() == ['b', 'a', 'c']
+
+    def test_default_name(self, table_types):
+        t = table_types.Table()
+        col = table_types.Column([1, 2, 3])
+        t.add_column(col)
+        assert t.columns.keys() == ['col0']
+
+
 @pytest.mark.usefixtures('table_types')
 class TestInitFromTable(SetupData):
 
@@ -486,6 +532,21 @@ class TestAddColumns(SetupData):
         t.add_columns([self.c, self.d], indexes=[2, 2])
         assert t.colnames == ['a', 'b', 'c', 'd']
 
+    def test_add_columns6(self, table_types):
+        """Check that we can override column names."""
+        self._setup(table_types)
+        t = table_types.Table()
+        t.add_columns([self.a, self.b, self.c], names=['b', 'c', 'a'])
+        assert t.colnames == ['b', 'c', 'a']
+
+    def test_add_columns7(self, table_types):
+        """Check that default names are used when appropriate."""
+        t = table_types.Table()
+        col0 = table_types.Column([1, 2, 3])
+        col1 = table_types.Column([4, 5, 3])
+        t.add_columns([col0, col1])
+        assert t.colnames == ['col0', 'col1']
+
     def test_add_duplicate_column(self, table_types):
         self._setup(table_types)
         t = table_types.Table()
@@ -515,6 +576,11 @@ class TestAddColumns(SetupData):
         # Check new column didn't change (since name conflict forced a copy)
         assert t['a_3'][0] == self.a[0]
 
+        # Check that rename_duplicate=True is ok if there are no duplicates
+        t.add_column(table_types.Column(name='q', data=[0, 1, 2]),
+                     rename_duplicate=True)
+        assert t.colnames == ['a', 'a_1', 'b', 'c', 'a_2', 'a_3', 'q']
+
     def test_add_duplicate_columns(self, table_types):
         self._setup(table_types)
         t = table_types.Table([self.a, self.b, self.c])
@@ -563,11 +629,11 @@ class TestAddRow(SetupData):
         t = table_types.Table(names=('a', 'b', 'c'), dtype=('(2,)i', 'S4', 'O'))
         t.add_row()
         assert np.all(t['a'][0] == [0, 0])
-        assert t['b'][0] == b''
+        assert t['b'][0] == ''
         assert t['c'][0] == 0
         t.add_row()
         assert np.all(t['a'][1] == [0, 0])
-        assert t['b'][1] == b''
+        assert t['b'][1] == ''
         assert t['c'][1] == 0
 
     def test_add_stuff_to_empty_table(self, table_types):
@@ -575,13 +641,13 @@ class TestAddRow(SetupData):
         t = table_types.Table(names=('a', 'b', 'obj'), dtype=('(2,)i', 'S8', 'O'))
         t.add_row([[1, 2], 'hello', 'world'])
         assert np.all(t['a'][0] == [1, 2])
-        assert t['b'][0] == b'hello'
+        assert t['b'][0] == 'hello'
         assert t['obj'][0] == 'world'
         # Make sure it is not repeating last row but instead
         # adding zeros (as documented)
         t.add_row()
         assert np.all(t['a'][1] == [0, 0])
-        assert t['b'][1] == b''
+        assert t['b'][1] == ''
         assert t['obj'][1] == 0
 
     def test_add_table_row(self, table_types):
@@ -605,6 +671,12 @@ class TestAddRow(SetupData):
         assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 4.0]))
         assert np.all(t['obj'] == np.array([1, 'string', 3, [10]], dtype='O'))
 
+    def test_add_qtable_row_multidimensional(self):
+        q = [[1, 2], [3, 4]] * u.m
+        qt = table.QTable([q])
+        qt.add_row(([5, 6] * u.km,))
+        assert np.all(qt['col0'] == [[1, 2], [3, 4], [5000, 6000]] * u.m)
+
     def test_add_with_tuple(self, table_types):
         self._setup(table_types)
         t = self.t
@@ -668,14 +740,14 @@ class TestAddRow(SetupData):
         t_copy = table_types.Table(t, copy=True)
         # Wrong number of columns
         try:
-            t.add_row([1,2,3,4])
+            t.add_row([1, 2, 3, 4])
         except ValueError:
             pass
         assert len(t) == 3
         assert np.all(t.as_array() == t_copy.as_array())
         # Wrong data type
         try:
-            t.add_row(['one',2,3])
+            t.add_row(['one', 2, 3])
         except ValueError:
             pass
         assert len(t) == 3
@@ -947,6 +1019,10 @@ class TestSort():
         assert np.all(t['x'] == x[idx])
         assert np.all(t['y'] == y[idx])
 
+    def test_empty(self, table_types):
+        t = table_types.Table([[], []], dtype=['f4', 'U1'])
+        t.sort('col1')
+
     def test_multiple(self, table_types):
         t = table_types.Table()
         t.add_column(table_types.Column(name='a', data=[2, 1, 3, 2, 3, 1]))
@@ -959,13 +1035,16 @@ class TestSort():
         t.sort(['b', 'a'])
         assert np.all(t['a'] == np.array([2, 1, 3, 1, 3, 2]))
         assert np.all(t['b'] == np.array([3, 4, 4, 5, 5, 6]))
+        t.sort(('a', 'b'))
+        assert np.all(t['a'] == np.array([1, 1, 2, 2, 3, 3]))
+        assert np.all(t['b'] == np.array([4, 5, 3, 6, 4, 5]))
 
     def test_multiple_with_bytes(self, table_types):
         t = table_types.Table()
         t.add_column(table_types.Column(name='firstname', data=[b"Max", b"Jo", b"John"]))
         t.add_column(table_types.Column(name='name', data=[b"Miller", b"Miller", b"Jackson"]))
         t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
-        t.sort(['name','firstname'])
+        t.sort(['name', 'firstname'])
         assert np.all([t['firstname'] == np.array([b"John", b"Jo", b"Max"])])
         assert np.all([t['name'] == np.array([b"Jackson", b"Miller", b"Miller"])])
         assert np.all([t['tel'] == np.array([19, 15, 12])])
@@ -981,7 +1060,7 @@ class TestSort():
             name='name',
             data=[six.text_type(x) for x in ["Miller", "Miller", "Jackson"]]))
         t.add_column(table_types.Column(name='tel', data=[12, 15, 19]))
-        t.sort(['name','firstname'])
+        t.sort(['name', 'firstname'])
         assert np.all([t['firstname'] == np.array(
             [six.text_type(x) for x in ["John", "Jo", "Max"]])])
         assert np.all([t['name'] == np.array(
@@ -1084,7 +1163,7 @@ class TestConvertNumpyArray():
         np_data = np.array(d)
         if table_types.Table is not MaskedTable:
             assert np.all(np_data == d.as_array())
-        assert not np_data is d.as_array()
+        assert np_data is not d.as_array()
         assert d.colnames == list(np_data.dtype.names)
 
         np_data = np.array(d, copy=False)
@@ -1174,6 +1253,7 @@ def test_copy_protocol():
     _assert_copies(t, t2, deep=False)
     _assert_copies(t, t3)
 
+
 def test_disallow_inequality_comparisons():
     """
     Regression test for #828 - disallow comparison operators on whole Table
@@ -1193,6 +1273,7 @@ def test_disallow_inequality_comparisons():
     with pytest.raises(TypeError):
         t <= -1.1
 
+
 def test_equality():
 
     t = table.Table.read([' a b  c  d',
@@ -1207,16 +1288,16 @@ def test_equality():
                          ], format='ascii')
 
     # All rows are equal
-    assert np.all(t==t)
+    assert np.all(t == t)
 
     # Assert no rows are different
-    assert not np.any(t!=t)
+    assert not np.any(t != t)
 
     # Check equality result for a given row
-    assert np.all((t == t[3]) == np.array([0,0,0,1,0,0,0,0], dtype=bool))
+    assert np.all((t == t[3]) == np.array([0, 0, 0, 1, 0, 0, 0, 0], dtype=bool))
 
     # Check inequality result for a given row
-    assert np.all((t != t[3]) == np.array([1,1,1,0,1,1,1,1], dtype=bool))
+    assert np.all((t != t[3]) == np.array([1, 1, 1, 0, 1, 1, 1, 1], dtype=bool))
 
     t2 = table.Table.read([' a b  c  d',
                            ' 2 c 7.0 0',
@@ -1231,12 +1312,12 @@ def test_equality():
 
     # In the above cases, Row.__eq__ gets called, but now need to make sure
     # Table.__eq__ also gets called.
-    assert np.all((t == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
-    assert np.all((t != t2) == np.array([0,0,1,0,1,0,1,0], dtype=bool))
+    assert np.all((t == t2) == np.array([1, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
+    assert np.all((t != t2) == np.array([0, 0, 1, 0, 1, 0, 1, 0], dtype=bool))
 
     # Check that comparing to a structured array works
-    assert np.all((t == t2.as_array()) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
-    assert np.all((t.as_array() == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t == t2.as_array()) == np.array([1, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
+    assert np.all((t.as_array() == t2) == np.array([1, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
 
 
 def test_equality_masked():
@@ -1256,16 +1337,16 @@ def test_equality_masked():
     t = table.Table(t, masked=True)
 
     # All rows are equal
-    assert np.all(t==t)
+    assert np.all(t == t)
 
     # Assert no rows are different
-    assert not np.any(t!=t)
+    assert not np.any(t != t)
 
     # Check equality result for a given row
-    assert np.all((t == t[3]) == np.array([0,0,0,1,0,0,0,0], dtype=bool))
+    assert np.all((t == t[3]) == np.array([0, 0, 0, 1, 0, 0, 0, 0], dtype=bool))
 
     # Check inequality result for a given row
-    assert np.all((t != t[3]) == np.array([1,1,1,0,1,1,1,1], dtype=bool))
+    assert np.all((t != t[3]) == np.array([1, 1, 1, 0, 1, 1, 1, 1], dtype=bool))
 
     t2 = table.Table.read([' a b  c  d',
                            ' 2 c 7.0 0',
@@ -1280,16 +1361,16 @@ def test_equality_masked():
 
     # In the above cases, Row.__eq__ gets called, but now need to make sure
     # Table.__eq__ also gets called.
-    assert np.all((t == t2) == np.array([1,1,0,1,0,1,0,1], dtype=bool))
-    assert np.all((t != t2) == np.array([0,0,1,0,1,0,1,0], dtype=bool))
+    assert np.all((t == t2) == np.array([1, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
+    assert np.all((t != t2) == np.array([0, 0, 1, 0, 1, 0, 1, 0], dtype=bool))
 
     # Check that masking a value causes the row to differ
     t.mask['a'][0] = True
-    assert np.all((t == t2) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
-    assert np.all((t != t2) == np.array([1,0,1,0,1,0,1,0], dtype=bool))
+    assert np.all((t == t2) == np.array([0, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
+    assert np.all((t != t2) == np.array([1, 0, 1, 0, 1, 0, 1, 0], dtype=bool))
 
     # Check that comparing to a structured array works
-    assert np.all((t == t2.as_array()) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t == t2.as_array()) == np.array([0, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
 
 
 @pytest.mark.xfail
@@ -1325,7 +1406,7 @@ def test_equality_masked_bug():
                            ' 1 a 1.0 7',
                           ], format='ascii')
 
-    assert np.all((t.as_array() == t2) == np.array([0,1,0,1,0,1,0,1], dtype=bool))
+    assert np.all((t.as_array() == t2) == np.array([0, 1, 0, 1, 0, 1, 0, 1], dtype=bool))
 
 
 # Check that the meta descriptor is working as expected. The MetaBaseTest class
@@ -1397,8 +1478,8 @@ def test_unicode_bytestring_conversion(table_types):
     assert t1['col0'].dtype.kind == 'S'
     assert t1['col1'].dtype.kind == 'S'
     assert t1['col2'].dtype.kind == 'i'
-    assert t1['col0'][0] == 'abc'.encode('ascii')
-    assert t1['col1'][0] == 'def'.encode('ascii')
+    assert t1['col0'][0] == 'abc'
+    assert t1['col1'][0] == 'def'
     assert t1['col2'][0] == 1
 
     t1 = t.copy()
@@ -1435,6 +1516,7 @@ def test_table_deletion():
 
     assert the_id in deleted
 
+
 def test_nested_iteration():
     """
     Regression test for issue 3358 where nested iteration over a single table fails.
@@ -1467,22 +1549,22 @@ class TestPandas(object):
 
         for endian in ['<', '>']:
             for kind in ['f', 'i']:
-                for byte in ['2','4','8']:
+                for byte in ['2', '4', '8']:
                     dtype = np.dtype(endian + kind + byte)
-                    x = np.array([1,2,3], dtype=dtype)
+                    x = np.array([1, 2, 3], dtype=dtype)
                     t[endian + kind + byte] = x
 
-        t['u'] = ['a','b','c']
-        t['s'] = [b'a', b'b', b'c']
+        t['u'] = ['a', 'b', 'c']
+        t['s'] = ['a', 'b', 'c']
 
         d = t.to_pandas()
 
         for column in t.columns:
             if column == 'u':
-                assert np.all(t['u'] == np.array(['a','b','c']))
+                assert np.all(t['u'] == np.array(['a', 'b', 'c']))
                 assert d[column].dtype == np.dtype("O")  # upstream feature of pandas
             elif column == 's':
-                assert np.all(t['s'] == np.array([b'a',b'b',b'c']))
+                assert np.all(t['s'] == np.array(['a', 'b', 'c']))
                 assert d[column].dtype == np.dtype("O")  # upstream feature of pandas
             else:
                 # We should be able to compare exact values here
@@ -1492,13 +1574,12 @@ class TestPandas(object):
                 else:
                     assert d[column].dtype == t[column].byteswap().newbyteorder().dtype
 
-
         # Regression test for astropy/astropy#1156 - the following code gave a
         # ValueError: Big-endian buffer not supported on little-endian
         # compiler. We now automatically swap the endian-ness to native order
         # upon adding the arrays to the data frame.
-        d[['<i4','>i4']]
-        d[['<f4','>f4']]
+        d[['<i4', '>i4']]
+        d[['<f4', '>f4']]
 
         t2 = table.Table.from_pandas(d)
 
@@ -1515,8 +1596,8 @@ class TestPandas(object):
     def test_2d(self):
 
         t = table.Table()
-        t['a'] = [1,2,3]
-        t['b'] = np.ones((3,2))
+        t['a'] = [1, 2, 3]
+        t['b'] = np.ones((3, 2))
 
         with pytest.raises(ValueError) as exc:
             t.to_pandas()
@@ -1527,7 +1608,7 @@ class TestPandas(object):
         from ...coordinates import SkyCoord
 
         t = table.Table()
-        t['c'] = SkyCoord([1,2,3], [4,5,6], unit='deg')
+        t['c'] = SkyCoord([1, 2, 3], [4, 5, 6], unit='deg')
 
         with pytest.raises(ValueError) as exc:
             t.to_pandas()
@@ -1543,10 +1624,10 @@ class TestPandas(object):
         t['b'] = [1., 2., 3.]
         t['b'].mask = [False, False, True]
 
-        t['u'] = ['a','b','c']
+        t['u'] = ['a', 'b', 'c']
         t['u'].mask = [False, True, False]
 
-        t['s'] = [b'a', b'b', b'c']
+        t['s'] = ['a', 'b', 'c']
         t['s'].mask = [False, True, False]
 
         d = t.to_pandas()
@@ -1609,6 +1690,77 @@ class TestReplaceColumn(SetupData):
             t.replace_column('a', [1, 2, 3])
         assert err.value.args[0] == 'cannot replace a table index column'
 
+
+class Test__Astropy_Table__():
+    """
+    Test initializing a Table subclass from a table-like object that
+    implements the __astropy_table__ interface method.
+    """
+
+    class SimpleTable(object):
+        def __init__(self):
+            self.columns = [[1, 2, 3],
+                            [4, 5, 6],
+                            [7, 8, 9] * u.m]
+            self.names = ['a', 'b', 'c']
+            self.meta = OrderedDict([('a', 1), ('b', 2)])
+
+        def __astropy_table__(self, cls, copy, **kwargs):
+            a, b, c = self.columns
+            c.info.name = 'c'
+            cols = [table.Column(a, name='a'),
+                    table.MaskedColumn(b, name='b'),
+                    c]
+            names = [col.info.name for col in cols]
+            return cls(cols, names=names, copy=copy, meta=kwargs or self.meta)
+
+    def test_simple_1(self):
+        """Make a SimpleTable and convert to Table, QTable with copy=False, True"""
+        for table_cls in (table.Table, table.QTable):
+            col_c_class = u.Quantity if table_cls is table.QTable else table.MaskedColumn
+            for cpy in (False, True):
+                st = self.SimpleTable()
+                # Test putting in a non-native kwarg `extra_meta` to Table initializer
+                t = table_cls(st, copy=cpy, extra_meta='extra!')
+                assert t.colnames == ['a', 'b', 'c']
+                assert t.meta == {'extra_meta': 'extra!'}
+                assert np.all(t['a'] == st.columns[0])
+                assert np.all(t['b'] == st.columns[1])
+                vals = t['c'].value if table_cls is table.QTable else t['c']
+                assert np.all(st.columns[2].value == vals)
+
+                assert isinstance(t['a'], table.MaskedColumn)
+                assert isinstance(t['b'], table.MaskedColumn)
+                assert isinstance(t['c'], col_c_class)
+                assert t['c'].unit is u.m
+                assert type(t) is table_cls
+
+                # Copy being respected?
+                t['a'][0] = 10
+                assert st.columns[0][0] == 1 if cpy else 10
+
+    def test_simple_2(self):
+        """Test converting a SimpleTable and changing column names and types"""
+        st = self.SimpleTable()
+        dtypes = [np.int32, np.float32, np.float16]
+        names = ['a', 'b', 'c']
+        t = table.Table(st, dtype=dtypes, names=names, meta=OrderedDict([('c', 3)]))
+        assert t.colnames == names
+        assert all(col.dtype.type is dtype
+                   for col, dtype in zip(t.columns.values(), dtypes))
+
+        # The supplied meta is ignored.  This is consistent with current
+        # behavior when initializing from an existing astropy Table.
+        assert t.meta == st.meta
+
+    def test_kwargs_exception(self):
+        """If extra kwargs provided but without initializing with a table-like
+        object, exception is raised"""
+        with pytest.raises(TypeError) as err:
+            table.Table([[1]], extra_meta='extra!')
+        assert '__init__() got unexpected keyword argument' in str(err)
+
+
 def test_replace_column_qtable():
     """Replace existing Quantity column with a new column in a QTable"""
     a = [1, 2, 3] * u.m
@@ -1627,3 +1779,179 @@ def test_replace_column_qtable():
     assert t.colnames == ['a', 'b']
     assert t['a'].info.meta is None
     assert t['a'].info.format is None
+
+
+def test_replace_update_column_via_setitem():
+    """
+    Test table update like ``t['a'] = value``.  This leverages off the
+    already well-tested ``replace_column`` and in-place update
+    ``t['a'][:] = value``, so this testing is fairly light.
+    """
+    a = [1, 2] * u.m
+    b = [3, 4]
+    t = table.QTable([a, b], names=['a', 'b'])
+    assert isinstance(t['a'], u.Quantity)
+
+    # Inplace update
+    ta = t['a']
+    t['a'] = 5 * u.m
+    assert np.all(t['a'] == [5, 5] * u.m)
+    assert t['a'] is ta
+
+    # Replace
+    t['a'] = [5, 6]
+    assert np.all(t['a'] == [5, 6])
+    assert isinstance(t['a'], table.Column)
+    assert t['a'] is not ta
+
+
+def test_replace_update_column_via_setitem_warnings_normal():
+    """
+    Test warnings related to table replace change in #5556:
+    Normal warning-free replace
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = 0  # in-place update
+            assert len(w) == 0
+
+            t['a'] = [10, 20, 30]  # replace column
+            assert len(w) == 0
+
+
+def test_replace_update_column_via_setitem_warnings_slice():
+    """
+    Test warnings related to table replace change in #5556:
+    Replace a slice, one warning.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t2 = t[:2]
+
+            t2['a'] = 0  # in-place slice update
+            assert np.all(t['a'] == [0, 0, 3])
+            assert len(w) == 0
+
+            t2['a'] = [10, 20]  # replace slice
+            assert len(w) == 1
+            assert "replaced column 'a' which looks like an array slice" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_attributes():
+    """
+    Test warnings related to table replace change in #5556:
+    Lost attributes.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    t['a'].unit = 'm'
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = [10, 20, 30]
+        assert len(w) == 1
+        assert "replaced column 'a' and column attributes ['unit']" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_refcount():
+    """
+    Test warnings related to table replace change in #5556:
+    Reference count changes.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    ta = t['a']  # Generate an extra reference to original column
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = [10, 20, 30]
+        assert len(w) == 1
+        assert "replaced column 'a' and the number of references" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_always():
+    """
+    Test warnings related to table replace change in #5556:
+    Test 'always' setting that raises warning for any replace.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings', ['always']):
+            t['a'] = 0  # in-place slice update
+            assert len(w) == 0
+
+            from inspect import currentframe, getframeinfo
+            frameinfo = getframeinfo(currentframe())
+            t['a'] = [10, 20, 30]  # replace column
+            assert len(w) == 1
+            assert "replaced column 'a'" == str(w[0].message)
+
+            # Make sure the warning points back to the user code line
+            assert w[0].lineno == frameinfo.lineno + 1
+            assert w[0].category is table.TableReplaceWarning
+            assert 'test_table' in w[0].filename
+
+
+def test_replace_update_column_via_setitem_replace_inplace():
+    """
+    Test the replace_inplace config option related to #5556.  In this
+    case no replace is done.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    ta = t['a']
+    t['a'].unit = 'm'
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_inplace', True):
+            with table.conf.set_temp('replace_warnings',
+                                     ['always', 'refcount', 'attributes', 'slice']):
+                t['a'] = 0  # in-place update
+                assert len(w) == 0
+                assert ta is t['a']
+
+                t['a'] = [10, 20, 30]  # normally replaces column, but not now
+                assert len(w) == 0
+                assert ta is t['a']
+                assert np.all(t['a'] == [10, 20, 30])
+
+
+def test_primary_key_is_inherited():
+    """Test whether a new Table inherits the primary_key attribute from
+    its parent Table. Issue #4672"""
+
+    t = table.Table([(2, 3, 2, 1), (8, 7, 6, 5)], names=('a', 'b'))
+    t.add_index('a')
+    original_key = t.primary_key
+
+    # can't test if tuples are equal, so just check content
+    assert original_key[0] is 'a'
+
+    t2 = t[:]
+    t3 = t.copy()
+    t4 = table.Table(t)
+
+    # test whether the reference is the same in the following
+    assert original_key == t2.primary_key
+    assert original_key == t3.primary_key
+    assert original_key == t4.primary_key
+
+    # just test one element, assume rest are equal if assert passes
+    assert t.loc[1] == t2.loc[1]
+    assert t.loc[1] == t3.loc[1]
+    assert t.loc[1] == t4.loc[1]
+
+
+def test_qtable_read_for_ipac_table_with_char_columns():
+    '''Test that a char column of a QTable is assigned no unit and not
+    a dimensionless unit, otherwise conversion of reader output to
+    QTable fails.'''
+    t1 = table.QTable([["A"]], names="B")
+    out = StringIO()
+    t1.write(out, format="ascii.ipac")
+    t2 = table.QTable.read(out.getvalue(), format="ascii.ipac", guess=False)
+    assert t2["B"].unit is None
