@@ -4,7 +4,6 @@
 
 from __future__ import print_function
 
-
 import functools
 import inspect
 import textwrap
@@ -12,14 +11,17 @@ import types
 import warnings
 
 from .codegen import make_function_with_signature
-from .exceptions import (AstropyDeprecationWarning,
+from .exceptions import (AstropyDeprecationWarning, AstropyUserWarning,
                          AstropyPendingDeprecationWarning)
 from ..extern import six
+from ..extern.six.moves import zip
 
 
-__all__ = ['deprecated', 'deprecated_attribute', 'classproperty',
-           'lazyproperty', 'sharedmethod', 'wraps',
-           'format_doc']
+__all__ = ['classproperty', 'deprecated', 'deprecated_attribute',
+           'deprecated_renamed_argument', 'format_doc',
+           'lazyproperty', 'sharedmethod', 'wraps']
+
+_NotFound = object()
 
 
 def deprecated(since, message='', name='', alternative='', pending=False,
@@ -78,9 +80,9 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         if not old_doc:
             old_doc = ''
         old_doc = textwrap.dedent(old_doc).strip('\n')
-        new_doc = (('\n.. deprecated:: %(since)s'
-                    '\n    %(message)s\n\n' %
-                    {'since': since, 'message': message.strip()}) + old_doc)
+        new_doc = (('\n.. deprecated:: {since}'
+                    '\n    {message}\n\n'.format(
+                    **{'since': since, 'message': message.strip()})) + old_doc)
         if not old_doc:
             # This is to prevent a spurious 'unexpected unindent' warning from
             # docutils when the original docstring was blank.
@@ -123,7 +125,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         # functools.wraps on it, but we normally don't care.
         # This crazy way to get the type of a wrapper descriptor is
         # straight out of the Python 3.3 inspect module docs.
-        if type(func) != type(str.__dict__['__add__']):
+        if type(func) is not type(str.__dict__['__add__']):  # nopep8
             deprecated_func = functools.wraps(func)(deprecated_func)
 
         deprecated_func.__doc__ = deprecate_doc(
@@ -181,21 +183,21 @@ def deprecated(since, message='', name='', alternative='', pending=False,
             name = get_function(obj).__name__
 
         altmessage = ''
-        if not message or type(message) == type(deprecate):
+        if not message or type(message) is type(deprecate):
             if pending:
-                message = ('The %(func)s %(obj_type)s will be deprecated in a '
+                message = ('The {func} {obj_type} will be deprecated in a '
                            'future version.')
             else:
-                message = ('The %(func)s %(obj_type)s is deprecated and may '
+                message = ('The {func} {obj_type} is deprecated and may '
                            'be removed in a future version.')
             if alternative:
-                altmessage = '\n        Use %s instead.' % alternative
+                altmessage = '\n        Use {} instead.'.format(alternative)
 
-        message = ((message % {
+        message = ((message.format(**{
             'func': name,
             'name': name,
             'alternative': alternative,
-            'obj_type': obj_type_name}) +
+            'obj_type': obj_type_name})) +
             altmessage)
 
         if isinstance(obj, type):
@@ -203,7 +205,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         else:
             return deprecate_function(obj, message)
 
-    if type(message) == type(deprecate):
+    if type(message) is type(deprecate):
         return deprecate(message)
 
     return deprecate
@@ -272,6 +274,243 @@ def deprecated_attribute(name, since, message=None, alternative=None,
     return property(get, set, delete)
 
 
+def deprecated_renamed_argument(old_name, new_name, since,
+                                arg_in_kwargs=False, relax=False,
+                                pending=False):
+    """Deprecate a _renamed_ function argument.
+
+    The decorator assumes that the argument with the ``old_name`` was removed
+    from the function signature and the ``new_name`` replaced it at the
+    **same position** in the signature.  If the ``old_name`` argument is
+    given when calling the decorated function the decorator will catch it and
+    issue a deprecation warning and pass it on as ``new_name`` argument.
+
+    Parameters
+    ----------
+    old_name : str or list/tuple thereof
+        The old name of the argument.
+
+    new_name : str or list/tuple thereof
+        The new name of the argument.
+
+    since : str or number or list/tuple thereof
+        The release at which the old argument became deprecated.
+
+    arg_in_kwargs : bool or list/tuple thereof, optional
+        If the argument is not a named argument (for example it
+        was meant to be consumed by ``**kwargs``) set this to
+        ``True``.  Otherwise the decorator will throw an Exception
+        if the ``new_name`` cannot be found in the signature of
+        the decorated function.
+        Default is ``False``.
+
+    relax : bool or list/tuple thereof, optional
+        If ``False`` a ``TypeError`` is raised if both ``new_name`` and
+        ``old_name`` are given.  If ``True`` the value for ``new_name`` is used
+        and a Warning is issued.
+        Default is ``False``.
+
+    pending : bool or list/tuple thereof, optional
+        If ``True`` this will hide the deprecation warning and ignore the
+        corresponding ``relax`` parameter value.
+        Default is ``False``.
+
+    Raises
+    ------
+    TypeError
+        If the new argument name cannot be found in the function
+        signature and arg_in_kwargs was False or if it is used to
+        deprecate the name of the ``*args``-, ``**kwargs``-like arguments.
+        At runtime such an Error is raised if both the new_name
+        and old_name were specified when calling the function and
+        "relax=False".
+
+    Notes
+    -----
+    The decorator should be applied to a function where the **name**
+    of an argument was changed but it applies the same logic.
+
+    .. warning::
+        If ``old_name`` is a list or tuple the ``new_name`` and ``since`` must
+        also be a list or tuple with the same number of entries. ``relax`` and
+        ``arg_in_kwarg`` can be a single bool (applied to all) or also a
+        list/tuple with the same number of entries like ``new_name``, etc.
+
+    Examples
+    --------
+    The deprecation warnings are not shown in the following examples.
+
+    To deprecate a positional or keyword argument::
+
+        >>> from astropy.utils.decorators import deprecated_renamed_argument
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0')
+        ... def test(sigma):
+        ...     return sigma
+
+        >>> test(2)
+        2
+        >>> test(sigma=2)
+        2
+        >>> test(sig=2)
+        2
+
+    To deprecate an argument catched inside the ``**kwargs`` the
+    ``arg_in_kwargs`` has to be set::
+
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0',
+        ...                             arg_in_kwargs=True)
+        ... def test(**kwargs):
+        ...     return kwargs['sigma']
+
+        >>> test(sigma=2)
+        2
+        >>> test(sig=2)
+        2
+
+    By default providing the new and old keyword will lead to an Exception. If
+    a Warning is desired set the ``relax`` argument::
+
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0', relax=True)
+        ... def test(sigma):
+        ...     return sigma
+
+        >>> test(sig=2)
+        2
+
+    It is also possible to replace multiple arguments. The ``old_name``,
+    ``new_name`` and ``since`` have to be `tuple` or `list` and contain the
+    same number of entries::
+
+        >>> @deprecated_renamed_argument(['a', 'b'], ['alpha', 'beta'],
+        ...                              ['1.0', 1.2])
+        ... def test(alpha, beta):
+        ...     return alpha, beta
+
+        >>> test(a=2, b=3)
+        (2, 3)
+
+    In this case ``arg_in_kwargs`` and ``relax`` can be a single value (which
+    is applied to all renamed arguments) or must also be a `tuple` or `list`
+    with values for each of the arguments.
+
+    .. warning::
+        This decorator needs to access the original signature of the decorated
+        function. Therefore this decorator must be the **first** decorator on
+        any function if it needs to work for Python before version 3.4.
+    """
+    cls_iter = (list, tuple)
+    if isinstance(old_name, cls_iter):
+        n = len(old_name)
+        # Assume that new_name and since are correct (tuple/list with the
+        # appropriate length) in the spirit of the "consenting adults". But the
+        # optional parameters may not be set, so if these are not iterables
+        # wrap them.
+        if not isinstance(arg_in_kwargs, cls_iter):
+            arg_in_kwargs = [arg_in_kwargs] * n
+        if not isinstance(relax, cls_iter):
+            relax = [relax] * n
+        if not isinstance(pending, cls_iter):
+            pending = [pending] * n
+    else:
+        # To allow a uniform approach later on, wrap all arguments in lists.
+        n = 1
+        old_name = [old_name]
+        new_name = [new_name]
+        since = [since]
+        arg_in_kwargs = [arg_in_kwargs]
+        relax = [relax]
+        pending = [pending]
+
+    def decorator(function):
+        # Lazy import to avoid cyclic imports
+        from .compat.funcsigs import signature
+
+        # The named arguments of the function.
+        arguments = signature(function).parameters
+        keys = list(arguments.keys())
+        position = [None] * n
+
+        for i in range(n):
+            # Determine the position of the argument.
+            if new_name[i] in arguments:
+                param = arguments[new_name[i]]
+                # There are several possibilities now:
+
+                # 1.) Positional or keyword argument:
+                if param.kind == param.POSITIONAL_OR_KEYWORD:
+                    position[i] = keys.index(new_name[i])
+
+                # 2.) Keyword only argument (Python 3 only):
+                elif param.kind == param.KEYWORD_ONLY:
+                    # These cannot be specified by position.
+                    position[i] = None
+
+                # 3.) positional-only argument, varargs, varkwargs or some
+                #     unknown type:
+                else:
+                    raise TypeError('cannot replace argument "{0}" of kind '
+                                    '{1!r}.'.format(new_name[i], param.kind))
+
+            # In case the argument is not found in the list of arguments
+            # the only remaining possibility is that it should be catched
+            # by some kind of **kwargs argument.
+            # This case has to be explicitly specified, otherwise throw
+            # an exception!
+            elif arg_in_kwargs[i]:
+                position[i] = None
+            else:
+                raise TypeError('"{}" was not specified in the function '
+                                'signature. If it was meant to be part of '
+                                '"**kwargs" then set "arg_in_kwargs" to "True"'
+                                '.'.format(new_name[i]))
+
+        @functools.wraps(function)
+        def wrapper(*args, **kwargs):
+            for i in range(n):
+                # The only way to have oldkeyword inside the function is
+                # that it is passed as kwarg because the oldkeyword
+                # parameter was renamed to newkeyword.
+                if old_name[i] in kwargs:
+                    value = kwargs.pop(old_name[i])
+                    # Display the deprecation warning only when it's only
+                    # pending.
+                    if not pending[i]:
+                        warnings.warn(
+                            '"{0}" was deprecated in version {1} '
+                            'and will be removed in a future version. '
+                            'Use argument "{2}" instead.'
+                            ''.format(old_name[i], since[i], new_name[i]),
+                            AstropyDeprecationWarning, stacklevel=2)
+
+                    # Check if the newkeyword was given as well.
+                    newarg_in_args = (position[i] is not None and
+                                      len(args) > position[i])
+                    newarg_in_kwargs = new_name[i] in kwargs
+
+                    if newarg_in_args or newarg_in_kwargs:
+                        if not pending[i]:
+                            # If both are given print a Warning if relax is
+                            # True or raise an Exception is relax is False.
+                            if relax[i]:
+                                warnings.warn(
+                                    '"{0}" and "{1}" keywords were set. '
+                                    'Using the value of "{1}".'
+                                    ''.format(old_name[i], new_name[i]),
+                                    AstropyUserWarning)
+                            else:
+                                raise TypeError(
+                                    'cannot specify both "{}" and "{}"'
+                                    '.'.format(old_name[i], new_name[i]))
+                    else:
+                        # If the new argument isn't specified just pass the old
+                        # one with the name of the new argument to the function
+                        kwargs[new_name[i]] = value
+            return function(*args, **kwargs)
+
+        return wrapper
+    return decorator
+
+
 # TODO: This can still be made to work for setters by implementing an
 # accompanying metaclass that supports it; we just don't need that right this
 # second
@@ -493,11 +732,13 @@ class lazyproperty(property):
 
     def __get__(self, obj, owner=None):
         try:
-            return obj.__dict__[self._key]
-        except KeyError:
-            val = self.fget(obj)
-            obj.__dict__[self._key] = val
-            return val
+            val = obj.__dict__.get(self._key, _NotFound)
+            if val is not _NotFound:
+                return val
+            else:
+                val = self.fget(obj)
+                obj.__dict__[self._key] = val
+                return val
         except AttributeError:
             if obj is None:
                 return self
@@ -572,10 +813,6 @@ class sharedmethod(classmethod):
         this implements the Example.identify classmethod
     """
 
-    def __getobjwrapper(func):
-        return func
-
-    @__getobjwrapper
     def __get__(self, obj, objtype=None):
         if obj is None:
             mcls = type(objtype)
@@ -596,9 +833,7 @@ class sharedmethod(classmethod):
         else:
             return self._make_method(self.__func__, obj)
 
-    del __getobjwrapper
-
-    if six.PY3:
+    if not six.PY2:
         # The 'instancemethod' type of Python 2 and the method type of
         # Python 3 have slightly different constructors
         @staticmethod
@@ -641,11 +876,12 @@ def wraps(wrapped, assigned=functools.WRAPPER_ASSIGNMENTS,
     return wrapper
 
 
-if isinstance(wraps.__doc__, six.string_types):
+if (isinstance(wraps.__doc__, six.string_types) and
+        wraps.__doc__ is not None and functools.wraps.__doc__ is not None):
     wraps.__doc__ += functools.wraps.__doc__
 
 
-if six.PY3:
+if not six.PY2:
     def _get_function_args_internal(func):
         """
         Utility function for `wraps`.
@@ -794,7 +1030,7 @@ def format_doc(docstring, *args, **kwargs):
             num1, num2 : Numbers
             Returns
             -------
-            result: Number
+            result : Number
 
     in case one might want to format it further::
 
@@ -828,7 +1064,7 @@ def format_doc(docstring, *args, **kwargs):
             num1, num2 : Numbers
             Returns
             -------
-            result: Number
+            result : Number
                 result of num1 + num2
         >>> help(subtract) # doctest: +SKIP
         Help on function subtract in module __main__:
@@ -840,9 +1076,9 @@ def format_doc(docstring, *args, **kwargs):
             num1, num2 : Numbers
             Returns
             -------
-            result: Number
+            result : Number
                 result of num1 - num2
-            Notes: This one has additional notes.
+            Notes : This one has additional notes.
 
     These methods can be combined an even taking the docstring from another
     object is possible as docstring attribute. You just have to specify the
@@ -862,7 +1098,7 @@ def format_doc(docstring, *args, **kwargs):
             num1, num2 : Numbers
             Returns
             -------
-            result: Number
+            result : Number
                 result of num1 + num2
 
     But be aware that this decorator *only* formats the given docstring not
@@ -884,7 +1120,7 @@ def format_doc(docstring, *args, **kwargs):
             num1, num2 : Numbers
             Returns
             -------
-            result: Number
+            result : Number
                 result of num1 + num2
             This one is good for {0}.
 
